"""Example of the inundationmodel.

A wave of water is washed up ontp a hypothetical beach.

To run:

python beach.py
"""

######################
# Module imports 
#

import sys
from os import sep, path
sys.path.append('..'+sep+'pyvolution')

from shallow_water import Domain, Reflective_boundary, Dirichlet_boundary,\
     Transmissive_boundary, Time_boundary, Wind_stress

from pmesh2domain import pmesh_to_domain_instance
from config import data_dir
from util import read_polygon, Polygon_function
from math import pi
from Numeric import choose, greater, ones, sin, exp
import time

######################
# Domain
name = 'beach'
print 'Creating domain from %s.tsh' %name
domain = pmesh_to_domain_instance(name + '.tsh', Domain)

#HACK to remove internal boundary (until we get a solution)
#Is this still necessary?
for b in domain.boundary.keys():
    if domain.boundary[b] == '':
        del domain.boundary[b]

domain = Domain(domain.coordinates, domain.triangles, domain.boundary)

domain.store = True
domain.set_name(name)
print "Output being written to " + data_dir + sep + \
      domain.filename + "_size%d." %len(domain) + domain.format


def bathymetry(x, y):
    cut = 75
    
    res = choose( greater(x, cut), ( -(x - 55)/5, -4*ones(x.shape) )) 
    res += (100-y)/50 - 0.75
    return res
    

def topography(x, y):

    import RandomArray
    #z = 4*sin(pi*x/50) + (100-y)/50 + 1 #Ridge
    
    z = -4.0*x/25 + 8  + (100-y)/50 #Beach
    
    #z += sin(pi*x/5)/4 
    #z += RandomArray.normal(0, 1, x.shape)/4 
    
    z += 6*exp( -((x-30)/10)**2 ) * exp( -((y-50)/8)**2 )     #Mound
    z += 4*exp( -((x-30)/4)**2 ) * exp( -((y-26)/10)**2 )     #extra ridge    
    z += 4*exp( -((x-30)/5)**2 ) * exp( -((y-10)/8)**2 )     #extra ridge        
    
    return z

def riverbed(x, y):
    return (y-100)/70 - 4.0*x/25 + 8
    

shoreline = [[40,0], [100,0], [100,100], [65,100], 
             [55,90], [55,70], [56,50], [50,25], [40,0]]        

land = [[65,100], [55,90], [55,70], [56,50], [50,25], [40,0],
        [0,0], [0,100]] 
		       
	     
building1 = [[45,80], [49,78], [52,83], [46,83]]	     
building2 = [[35,75], [40,75], [40,80], [35,80]]	     
building3 = [[42,63], [46,61], [48,65], [44,67]]	     
building4 = [[28,56], [28,51], [33,51], [33,56]]	     
building5 = [[10,70], [10,65], [15,65], [15,70]]	     
building6 = [[10,50], [10,45], [15,45], [15,50]]	     

river = [[20,100], [18,90], [20,80], [20,60], [15,40], [11,20], [2,0], [10,0],
         [14,10], [20,30], [24,45], [27,80], [27,85], [35,90], [39,100]]
	      
print 'Set elevation'    
t0 = time.time()
domain.set_quantity('elevation',
    Polygon_function( [(shoreline, bathymetry), (land, topography), 
                      (building1, 6), (building2, 8), 
		      (building3, 6), (building4, 13),
		      (building5, 10), (building6, 11),
		      (river, riverbed)]))
		      
print 'That took %.2f seconds' %(time.time()-t0)
		      
print 'Set level'    		      
domain.set_quantity('level', 
		    Polygon_function( [(shoreline, -1.5), 
		    (land, -10)] )) 
		    
print 'Set friction'    		      		    
domain.set_quantity('friction', 0.075)

print domain.get_extent()



#Add lateral wind gusts bearing 135 degrees
def gust(t,x,y): 
    from math import sin, pi
    from Numeric import zeros, ones, Float

    N = len(x)

    tt = sin(2*pi*t/50)

    if tt > 0.7:
        return 30000*tt*ones(N, Float)
    else:
        return zeros(N, Float)
    
domain.forcing_terms.append(Wind_stress(gust, 135))


######################
# Boundary conditions

print 'Boundaries'
Br = Reflective_boundary(domain)
Bo = Transmissive_boundary(domain)

#Constant outflow
Bd = Dirichlet_boundary([-10, 0.0, 0.0])
Bt = Time_boundary(domain, lambda t: [ 2.8*(1+sin(2*pi*t/100)), 0.0, 0.0])

print 'Available boundary tags are', domain.get_boundary_tags()

#Set boundary conditions
tags = {}
tags['ocean'] = Bt
tags['wall'] = Br 
tags['wall1'] = Br 
tags['outflow'] = Bd 
tags['exterior'] = Br
tags['external'] = Br
tags['land'] = Bo  
tags['westbank'] = None    #Riverbank
tags['eastbank'] = None    #Riverbank
tags['eastbankN'] = None    #Riverbank
domain.set_boundary(tags)


domain.check_integrity()

######################
#Evolution
for t in domain.evolve(yieldstep = 0.2, finaltime = 300):
    domain.write_time()
    
print 'Done'

    
